#!/usr/bin/perl
##############################################################################
# Owner : Martin Rieder (martin.rieder@schwindt.eu)                          #
#                                                                            #
# Name               : sctliccheck.pl                                        #
# Intention          : Read License Usage into XML                           #
# Require            : perl                                                  #
#                                                                            #
# v1.00 04.05.15 mrd : created                                               #
# v2.00 10.09.15 mrd : some enhancements                                     #
# v2.10 22.09.15 mrd : local licenses added                                  #
##############################################################################

use strict;
use List::MoreUtils qw(firstidx uniq);
use XML::Simple;
use Getopt::Long;
use File::Spec::Functions;
my %opt;

my %months = ( 	Jan => '01',
				Feb => '02', 
				Mar => '03', 
				Apr => '04', 
				May => '05', 
				Jun => '06', 
				Jul => '07', 
				Aug => '08', 
				Sep => '09', 
				Oct => '10', 
				Nov => '11', 
				Dec => '12');
my $dsls_config;
my %dsls_server;
my @dsls_server_failover;
my @exclude;

my $dslicsrv_command="$ENV{CATPRO_LOC}\\bin\\DSLS\\win_b64\\code\\bin\\DSLicSrv.exe";
my $dslicmgtb_command;
my $dsls_admin_port="4084";

my (%count, %inuse, %usage, %expiration);
my @lic;
my %xml;

&GetOptions(\%opt, 'catinstallpath=s');

if ( defined($opt{catinstallpath}) ) {
	if ( -e (catfile($opt{catinstallpath}, "code", "bin", "DSLicMgtb.exe")) ) {
		$dslicmgtb_command=catfile($opt{catinstallpath}, "code", "bin", "DSLicMgtb.exe");
	}
}

if ( -e "$ENV{CABUM_LICCONF}" ) {
	&readconfig;
} else {
	print "Config-file not found!\n";
}

my $path=$ENV{CABUM_LICCONF_OUTPUT};
$path =~ s/\\/\//g;
foreach (glob("$path/cabum_licconf*.xml")) {
    unlink "$_";
}

if ($ENV{DSLS_CONFIG}) {
    $dsls_config=$ENV{DSLS_CONFIG};
} else {
    $dsls_config=$ENV{ALLUSERSPROFILE} . "\\DassaultSystemes\\Licenses\\DSLicSrv.txt";
}

if (@dsls_server_failover) {
	@dsls_server_failover=uniq(@dsls_server_failover);
    foreach (@dsls_server_failover) {
		my ($temp1, $temp2)=split(":", $_);
		$dsls_server{$temp1}=$temp2;
    }
} else {
    &get_license_server($dsls_config);
}

for (keys %dsls_server) {
    &get_license_info($_, $dsls_server{$_});
}

if ($dslicmgtb_command) {&get_license_info_local};

@lic=sort(@lic);

foreach (@lic) {
    my ($trigram,$server)=split(";", $_);
	my $trigram_real = $trigram;
	$trigram_real =~ s/_\d+//;
    my $ignore="no";
    foreach my $exclude (@exclude) {
		if ($exclude eq $trigram_real) {
		    $ignore="yes";
		}
    }
    if ($ignore eq "no") {
		if ($usage{$_}) {
			my @usage=split(/=/,$usage{$_});
			my %license_usage;
			for (my $i=0; $i<scalar(@usage); $i++) {
				my ($computer,$user,$process,$since,$until)=split(/;/,$usage[$i]);
				my ($computer_next,$user_next,$process_next,$since_next,$until_next)=split(/;/,$usage[$i+1]);
				if (($computer eq $computer_next) && ($user eq $user_next)) {
					$usage[$i]="$computer;$user;$process*$process_next;$since;$until";
					splice(@usage,$i+1,1);
					$i--;
				}
			}
			foreach (@usage) {
				my ($computer,$user,$process,$since,$until)=split(/;/,$_);
				my (@process)=split(/\*/,$process);
			    push(@{$license_usage{'license_usage'}},{
					'Server' 	=> $server,
					'Computer' 	=> $computer,
					'User' 		=> $user,
					'Process'	=> \@process,
					'Since' 	=> $since,
					'Until'		=> $until
				});
			}
		    push(@{$xml{'lic'}},{
		        'Name'		=> $trigram_real,
		        'Count'		=> $count{$_},
		        'In-Use'	=> $inuse{$_},
				'Server'	=> $server,
				%license_usage
			});
		} else {
			push(@{$xml{'lic'}},{
			    'Name'		=> $trigram_real,
			    'Count'		=> $count{$_},
			    'In-Use'	=> $inuse{$_},
				'Server'	=> $server,
			});
		}
    }
}

&xml_output;

sub readconfig {
	open(FILE,"$ENV{CABUM_LICCONF}");
	my @config=<FILE>;
	close(FILE);
	for (my $i=0;$i<@config;$i++) {
		if ( ($config[$i] !~ m/^#+/) && ($config[$i] !~ /^\s*$/) ) { 
			if ( not rindex($config[$i],"=") == -1 ) {
				$config[$i] =~ s/^\s+//;
				$config[$i] =~ s/\s+$//;
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/license-server/i ) {
					push(@dsls_server_failover, substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1)));
				}
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/admin-port/i ) {
					$dsls_admin_port = substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1));
				}
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/exclude/i ) {
					push(@exclude, split(",", substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1))));
				}
			}
		}
	}
}

sub get_license_server {
    open(FILE, $_[0]);
    my @dsls=<FILE>;
    close(FILE);
	@dsls=uniq(@dsls);
    foreach (@dsls) {
		chomp($_);
		if ($_ !~ m/^#/) {
		    (my $temp)=split(/:/, $_);
		    $dsls_server{$temp}=$dsls_admin_port;
		}
    }
}

sub get_license_info {
    my @temp1=`\"$dslicsrv_command\" -admin -run \"c $_[0] $_[1] -r; glu -all; ea -o "$ENV{CABUM_LICCONF_OUTPUT}\\cabum_licconf_rules_$_[0].xml"; d\" -locale en_US`;
    for(my $i = (firstidx {/^\tDassault Systemes V5/} @temp1)+1; $i < @temp1; $i++) {
		if ($temp1[$i] =~ m/^\t\t\w{3}/) {
		    chomp $temp1[$i];
		    my $trigram=substr($temp1[$i], 0, index($temp1[$i], "maxReleaseNumber"));
		    $trigram =~ s/\s//g;
			$trigram = $trigram . "_" . scalar(@lic);
		    my $expiration_date=substr($temp1[$i], index($temp1[$i], "expirationDate")+15, (index($temp1[$i], "type"))-(index($temp1[$i], "expirationDate")+15));
		    $expiration_date=~s/^\s*(.*)\s$/$1/;
		    my($month, $day, $year)=split(/ /, $expiration_date);
		    $day =~ s/,//g;
		    if ($day < 10) {$day="0" . $day};
		    $month=$months{$month};
		    $expiration_date="$year-$month-$day";
		    my $count=substr($temp1[$i], index($temp1[$i], "count")+6, (index($temp1[$i], "inuse"))-(index($temp1[$i], "count")+6));
		    $count =~ s/\s//g;
		    my $inuse=substr($temp1[$i], index($temp1[$i], "inuse")+6, (index($temp1[$i], "customerId"))-(index($temp1[$i], "inuse")+6));
		    $inuse =~ s/\s//g;
		    push (@lic, "$trigram;$_");
		    $count{"$trigram;$_"}=$count{"$trigram;$_"} + $count;
		    $inuse{"$trigram;$_"}=$inuse{"$trigram;$_"} + $inuse;
		    $expiration{"$trigram;$_"}="$count;$expiration_date";
		}
		if ($temp1[$i] =~ m/^\t{4}.*$/) {
			my $until = "";
		    chomp $temp1[$i];
		    $temp1[$i] =~ s/^\t{4}//;
		    my ($computer, $user);
		    ($computer,undef,$user)=split(/\s+/, $temp1[$i]);
		    my $process = substr($temp1[$i], index($temp1[$i], $user)+length($user), (index($temp1[$i], "granted"))-(index($temp1[$i], "$user")+length($user)));
		    $process=~s/^\s*//;
		    $process=~s/\s*$//;
			if ($process =~ /offline/) {$process="offline";}
		    my $since = substr($temp1[$i], index($temp1[$i], "since:")+7);
		    my($month, $day, $year, $time, $meridiem)=split(/ /, $since);
		    $day =~ s/,//g;
		    if ($day < 10) {$day="0" . $day};
		    $month=$months{$month};
		    my ($hours, $minutes) = split(/:/, $time);
		    if ($meridiem eq "PM" && $hours < 12) {$hours+=12};
		    if ($hours < 10) {$hours="0" . $hours};
		    $since="$year-$month-$day, $hours:$minutes";
			if ($process eq "offline") {
				$until = substr($temp1[$i], index($temp1[$i], "hold until:")+12);
			    ($month, $day, $year, $time, $meridiem)=split(/ /, $until);
			    $day =~ s/,//g;
			    if ($day < 10) {$day="0" . $day};
			    $month=$months{$month};
			    my ($hours, $minutes) = split(/:/, $time);
			    if ($meridiem eq "PM" && $hours < 12) {$hours+=12};
			    if ($hours < 10) {$hours="0" . $hours};
			    $until="$year-$month-$day, $hours:$minutes";
			}
			$usage{$lic[-1]}=$usage{$lic[-1]} . "$computer;$user;$process;$since;$until=";
		}
    }
}

sub get_license_info_local {
    my @temp1=`\"$dslicmgtb_command\"`;
    foreach(@temp1) {
	    chomp $_;
		if ($_) {
			my ($trigram,$server)=split(/-/,$_);
			chomp $trigram;
			chomp $server;
			$trigram =~ s/\s//g;
			$server =~ s/\s//g;
			$trigram = $trigram . " (" . $server . ")";
			$trigram = $trigram . "_" . scalar(@lic);
			push (@lic, "$trigram;$server");
		    $count{"$trigram;$server"}="1";
		    $inuse{"$trigram;$server"}="-";
		}
    }
}

sub xml_output {
    open(FILE, ">:encoding(UTF-8)", "$ENV{CABUM_LICCONF_OUTPUT}\\cabum_licconf.xml");
    my $xsimple = XML::Simple->new(ForceArray=>1,rootname=>'license_config',noattr=>1,xmldecl=>'<?xml version="1.0" encoding="utf-8"?>');
    print FILE $xsimple->XMLout({
		'licenses'	=> [\%xml],
		'timestamp'	=> time
    });
    close(FILE);
}
