#!/usr/bin/perl
##############################################################################
# Owner : Martin Rieder (martin.rieder@schwindt.eu)                          #
#                                                                            #
# Name               : sctrep.pl                                             #
# Intention          : Replication from CATREP_SERVER to clients             #
# Require            : perl                                                  #
#                                                                            #
# v1.00 23.08.12 mrd : created                                               #
# v2.00 07.10.14 mrd : scan-mode added                                       #
# v2.01 21.10.14 mrd : Bugfix (fdec\site\all not replicated)                 #
# v2.02 27.10.14 mrd : Bugfix in Delete-Subroutine                           #
# v2.03 03.11.14 mrd : Fix in Source-Routine (Multi-CatInstallPath)          #
# v2.04 20.04.15 mrd : More robust call of robocopy.exe						 #
##############################################################################

use strict;
use File::Glob;
use File::Path;
use File::Find;

my @robocopy;
my @source;
my @destination;
my @replace_source;
my @replace_destination;
my @env_files;
my @env_files_old;
my $i;
my $replication_default_mode="all";
my $replication_custom_mode="none";
my $replication_mode;
my $is_empty;

&setrobocopy;

if ( -e "$ENV{CABUM_REPCONF}" ) {
	&readconfig;
} else {
	print "Config-file not found!\n";
}

if ( -e "$ENV{CATMY_REF}\\cabum\\CatInstPath.txt" ) {
	&readconfig_path;
} else {
	print "CatInstPath.txt not found!\n";
}

if ( $replication_custom_mode !~ m/none/ ) { 
	$replication_mode = $replication_custom_mode;
} else {
	$replication_mode = $replication_default_mode;
}

if ( $replication_mode =~ m/all/ ) {
	print "\n*** Replication mode: all ***\n\n";
	push(@source,$ENV{CATPRO_REF});
	push(@destination,$ENV{CATPRO_LOC});
	push(@source,$ENV{CATSCT_REF_CATIA});
	push(@destination,$ENV{CATSCT_LOC_CATIA});
	push(@source,$ENV{CATSCT_REF_NX});
	push(@destination,$ENV{CATSCT_LOC_NX});
	push(@source,$ENV{CATMY_REF});
	push(@destination,$ENV{CATMY_LOC});
	push(@source,$ENV{CATCUS_REF});
	push(@destination,$ENV{CATCUS_LOC});
	push(@source,$ENV{FDEC_REF});
	push(@destination,$ENV{FDEC_LOC});
	&start_robocopy;
}

if ( $replication_mode =~ m/scan/) {
	print "\n*** Replication mode: scan ***\n\n";
	my $site_sct;
	my $site_my;
	my $exe_path;
	my $cad_type;
	my @replicate;
	my @delete;
	my $found;
	&collect_env_files;
	&collect_env_files_old;
	foreach (@env_files) {
		$found=0;
		($cad_type, $exe_path, $site_sct, $site_my) = &source_env($_);
		if ($cad_type eq "catia") {
			if (-e "$exe_path\\code\\bin\\cnext.exe") {
				$found=1;
			} else {
				for ($i=0;$i<@replace_source;$i++) {
					my $temp=$exe_path;
					$temp=~s/$replace_source[$i]/$replace_destination[$i]/g;
					if (-e "$temp\\code\\bin\\cnext.exe") {
						$found=1;
					}
				}
			}
		}
		if ($cad_type eq "nx") {
			if (-e "$exe_path\\ugraf.exe") {
				$found=1;
			} else {
				for ($i=0;$i<@replace_source;$i++) {
					my $temp=$exe_path;
					$temp=~s/$replace_source[$i]/$replace_destination[$i]/g;
					if (-e "$temp\\ugraf.exe") {
						$found=1;
					}
				}
			}
		}
		if ($found==0) {
			if ($site_sct) {push(@delete, $site_sct)};
			if ($site_my) {push(@delete, $site_my)};
		}
		if ($found==1) {
			if ($site_sct) {push(@replicate, $site_sct)};
			if ($site_my) {push(@replicate, $site_my)};
			$found=0;
		}
	}
	foreach (@env_files_old) {
		$found=0;
		my $env=$_;
		my $temp1=$ENV{CATREP_REF};
		$temp1 =~ s/\\/\//g;
		my $temp2=$ENV{CATREP_LOC};
		$temp2 =~ s/\\/\//g;
		$env =~ s/\\/\//g;
		$env =~ s/$temp1/$temp2/g;
		$env =~ s/\//\\/g;
		($cad_type, $exe_path, $site_sct, $site_my) = &source_env($_);
		if ($cad_type eq "catia") {
			if (-e "$exe_path\\code\\bin\\cnext.exe") {
				$found=1;
			} else {
				for ($i=0;$i<@replace_source;$i++) {
					my $temp=$exe_path;
					$temp=~s/$replace_source[$i]/$replace_destination[$i]/g;
					if (-e "$temp\\code\\bin\\cnext.exe") {
						$found=1;
					}
				}
			}
		}
		if ($found==0) {
			if ($env) {push(@delete, $env)};
			if ($site_sct) {push(@delete, $site_sct)};
			if ($site_my) {push(@delete, $site_my)};
		}
		if ($found==1) {
			if ($env) {push(@replicate, $env)};
			if ($site_sct) {push(@replicate, $site_sct)};
			if ($site_my) {push(@replicate, $site_my)};
			$found=0;
		}
	}
	splice(@replicate,0,@replicate,&remove_duplicates(@replicate));
	splice(@delete,0,@delete,&remove_duplicates(@delete));
	splice(@delete,0,@delete,&filter_delete(\@replicate,\@delete));
	
	push(@source,$ENV{CATPRO_REF});
	push(@destination,$ENV{CATPRO_LOC});
	push(@source,$ENV{CATMY_REF} . "\\cabum");
	push(@destination,$ENV{CATMY_LOC} . "\\cabum");
	push(@source,$ENV{CATMY_REF} . "\\othercad");
	push(@destination,$ENV{CATMY_LOC} . "\\othercad");
	push(@source,$ENV{CATMY_REF} . "\\catia\\all");
	push(@destination,$ENV{CATMY_LOC} . "\\catia\\all");
	push(@source,$ENV{CATMY_REF} . "\\nx\\all");
	push(@destination,$ENV{CATMY_LOC} . "\\nx\\all");
	push(@source,$ENV{CATCUS_REF} . "\\env");
	push(@destination,$ENV{CATCUS_LOC} . "\\env");
	push(@source,$ENV{FDEC_REF} . "\\env");
	push(@destination,$ENV{FDEC_LOC} . "\\env");
	push(@source,$ENV{FDEC_REF} . "\\site\\all");
	push(@destination,$ENV{FDEC_LOC} . "\\site\\all");
	foreach (@replicate) {
		push(@destination, $_);
		my $temp1=$ENV{CATSCT_LOC};
		$temp1 =~ s/\\/\//g;
		my $temp2=$ENV{CATSCT_REF};
		$temp2 =~ s/\\/\//g;
		my $temp3=$ENV{CATMY_LOC};
		$temp3 =~ s/\\/\//g;
		my $temp4=$ENV{CATMY_REF};
		$temp4 =~ s/\\/\//g;
		my $temp5=$ENV{CATCUS_LOC};
		$temp5 =~ s/\\/\//g;
		my $temp6=$ENV{CATCUS_REF};
		$temp6 =~ s/\\/\//g;
		my $temp7=$ENV{FDEC_LOC};
		$temp7 =~ s/\\/\//g;
		my $temp8=$ENV{FDEC_REF};
		$temp8 =~ s/\\/\//g;
		$_ =~ s/\\/\//g;
		$_ =~ s/$temp1/$temp2/g;
		$_ =~ s/$temp3/$temp4/g;
		$_ =~ s/$temp5/$temp6/g;
		$_ =~ s/$temp7/$temp8/g;
		$_ =~ s/\//\\/g;
		push(@source, $_);
	}
	&start_robocopy;
	&start_delete(@delete);
}

# Subprograms

sub start_robocopy {
	my @copy;
	$copy[0]="copy";
	$copy[3]="/y";
	$copy[4]="1>nul";
	$copy[5]="2>&1";
	for ($i=0;$i<@source;$i++) {
		if ($source[$i]=~m/\.\w+$/) {
			if (-e $source[$i]) {
				$copy[1]="$source[$i]";
				$copy[2]="$destination[$i]";
				print "Copy " . $source[$i] . "\n";
				system("cmd", "/c", @copy);
			}
		} else {
			if (-d $source[$i]) {
				$robocopy[2] = "\"" . $robocopy[3] . " - " . $source[$i] . "\"";
				$robocopy[4] = $source[$i];
				$robocopy[5] = $destination[$i];
				print "Replicate " . $source[$i] . "\n";
				system("cmd", "/c", @robocopy);
			}
		}
	}
}

sub start_delete {
	my @rd;
	$rd[0]="rd";
	$rd[2]="/s";
	$rd[3]="/q";
	$rd[4]="1>nul";
	$rd[5]="2>&1";
	foreach (@_) {
		if ($_=~m/\.\w+$/) {
			if (-e $_) {
				print "Deleting " . $_ . "\n";
				unlink $_;
			}
		} else {
			if (-d $_) {
				$rd[1]="$_";
				print "Deleting " . $_ . "\n";
				system("cmd", "/c", @rd);
			}
		}
		my $deleted=1;
		while ($deleted==1) {
			$_ = substr($_,0,rindex($_,"\\"));
			$is_empty=1;
			$deleted=0;
			eval {find(\&check_dir, $_)};
			if ($is_empty==1) {
				print "Deleting " . $_ . "\n";
				$rd[1]="$_";
				system("cmd", "/c", @rd);
				$deleted=1
			}
		}
	}
}

sub check_dir {
	if ($_ ne ".") {
		$is_empty=0;
		die;
	}
}

sub remove_duplicates {
	my @unique = ();
	my %seen   = ();

	foreach (@_) {
		next if $seen{$_}++;
		push @unique, $_;
	}
	return @unique;
}

sub filter_delete {
	my $replicate=shift;
	my $delete=shift;
	my %hashtemp;
	
	foreach (@$replicate) {$hashtemp{$_}++}
	
	for ($i=0;$i<@$delete;$i++) {
		if ($hashtemp{@$delete[$i]}) {
			splice(@$delete,$i,1);
		}
		
	}
	return @$delete;
}

sub collect_env_files {
	my $path=$ENV{CATSCT_REF};
	$path =~ s/\\/\//g;
	foreach (glob("$path/*/*/*/env/*.*")) {
		if ( substr($_,-3) ne "xml" && substr($_,-6) ne "sample" && substr($_,-4) ne "user" ) { push (@env_files, $_) };
	}
	$path=$ENV{CATMY_REF};
	$path =~ s/\\/\//g;
	foreach (glob("$path/*/*/*/env/*.*")) {
		if ( substr($_,-3) ne "xml" && substr($_,-6) ne "sample" && substr($_,-4) ne "user" ) { push (@env_files, $_) };
	}
}

sub collect_env_files_old {
	my $path=$ENV{CATCUS_REF};
	$path =~ s/\\/\//g;
	foreach (glob("$path/env/*.*")) {
		if ( substr($_,-3) ne "xml" && substr($_,-6) ne "sample" && substr($_,-4) ne "user" ) { push (@env_files_old, $_) };
	}
	$path=$ENV{FDEC_REF};
	$path =~ s/\\/\//g;
	foreach (glob("$path/env/*.*")) {
		if ( substr($_,-3) ne "xml" && substr($_,-6) ne "sample" && substr($_,-4) ne "user" ) { push (@env_files_old, $_) };
	}
}

sub source_env {
	open(FILE, $_[0]);
	my @env=<FILE>;
	close(FILE);
	my %environment;
	my %environment_filtered;
	my @env_filtered;
	my $left;
	my $right;
	for (keys %ENV) {
		$environment{$_} = $ENV{$_};
	}
	for ($i=0;$i<@env;$i++) {
		if ( ($env[$i] !~ m/^!+/) && ($env[$i] !~ /^\s*$/) ) {
			chomp($env[$i]);
			if ( not rindex($env[$i],"=") == -1 ) {
				push(@env_filtered,$env[$i]);
				$environment{substr($env[$i],0,rindex($env[$i],"="))}=substr($env[$i],rindex($env[$i],"=")+1,length($env[$i])-(rindex($env[$i],"=")+1));
			}
		}
	}
	for ($i=0;$i<@env_filtered;$i++) {
		($left, $right)=split("=",$env_filtered[$i]);
		$right =~ s/\%([^\%]+)\%/$environment{$1}/g;
		$env_filtered[$i]=$left . "=" . $right;
		$environment{$left}=$right;
		$environment_filtered{$left}=$right;
	}
	my $site_sct;
	my $site_my;
	my $exe_path;
	my $cad_type;
	if (exists($environment_filtered{"CAT_SITE"})) { $site_sct=$environment_filtered{"CAT_SITE"} };
	if (exists($environment_filtered{"NX_SITE"})) { $site_sct=$environment_filtered{"NX_SITE"} };
	if (exists($environment_filtered{"CAT_SITE_MY"})) { $site_my=$environment_filtered{"CAT_SITE_MY"} };
	if (exists($environment_filtered{"CAT_SITE_FDEC"})) { $site_my=$environment_filtered{"CAT_SITE_FDEC"} };
	if (exists($environment_filtered{"NX_SITE_MY"})) { $site_my=$environment_filtered{"NX_SITE_MY"} };
	if (exists($environment_filtered{"CATInstallPath"})) {
		$exe_path=$environment_filtered{"CATInstallPath"};
		($exe_path)=split(";",$exe_path);
		$exe_path=~s/\\/\//g;
		$cad_type="catia";
	}
	if (exists($environment_filtered{"UGII_ROOT_DIR"})) {
		$exe_path=$environment_filtered{"UGII_ROOT_DIR"};
		$exe_path=~s/\\/\//g;
		$cad_type="nx";
	}
	
	return($cad_type, $exe_path, $site_sct, $site_my);
}

sub readconfig {
	open(FILE,"$ENV{CABUM_REPCONF}");
	my @config=<FILE>;
	close(FILE);
	for ($i=0;$i<@config;$i++) {
		if ( ($config[$i] !~ m/^#+/) && ($config[$i] !~ /^\s*$/) ) { 
			if ( not rindex($config[$i],"=") == -1 ) {
				$config[$i] =~ s/^\s+//;
				$config[$i] =~ s/\s+$//;
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/default/i ) {
					$replication_default_mode = substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1));
				}
				if ( substr($config[$i],0,rindex($config[$i],"=")) =~ m/$ENV{COMPUTERNAME}/i ) {
					$replication_custom_mode = substr($config[$i],rindex($config[$i],"=")+1,length($config[$i])-(rindex($config[$i],"=")+1));
				}
			}
		}
	}
}

sub readconfig_path {
	open(FILE,"$ENV{CATMY_REF}\\cabum\\CatInstPath.txt");
	my @config=<FILE>;
	close(FILE);
	my $j = 0;
	for ($i=0;$i<@config;$i++) {
		if ( ($config[$i] !~ m/^#+/) && ($config[$i] !~ /^\s*$/) ) {
			($replace_source[$j], $replace_destination[$j]) = split ( ' ', $config[$i] );
			$replace_source[$j] =~ s/\\/\//g;
			$replace_destination[$j] =~ s/\\/\//g;
			$j++;
		}
	}
}

sub setrobocopy {
	$robocopy[0] = "start";
	$robocopy[1] = "/wait";

	if ( defined $ENV{osversion} && $ENV{osversion} >= 7 ) {
		if ( -e "$ENV{SystemRoot}\\system32\\robocopy.exe" ) { $robocopy[3] = $ENV{SystemRoot} ."\\system32\\robocopy.exe" };
		$i = 6;
		if ( defined $ENV{rc_mir} && $ENV{rc_mir} > 0 ) { 
			$robocopy[$i] =  "/MIR";
			$i++;
		}
		if ( defined $ENV{rc_retry} && $ENV{rc_retry} > 0 ) { 
			$robocopy[$i] =  "/R:" . $ENV{rc_retry};
			$i++;
		}
		if ( defined $ENV{rc_wait} && $ENV{rc_wait} > 0 ) { 
			$robocopy[$i] =  "/W:" . $ENV{rc_wait};
			$i++;
		}
		if ( defined $ENV{rc_multithread} && $ENV{rc_multithread} > 0 ) { 
			$robocopy[$i] =  "/MT:" . $ENV{rc_multithread};
			$i++;
		}
		if ( defined $ENV{rc_nfl} && $ENV{rc_nfl} > 0 ) { 
			$robocopy[$i] =  "/NFL";
			$i++;
		}
		if ( defined $ENV{rc_ndl} && $ENV{rc_ndl} > 0 ) { 
			$robocopy[$i] =  "/NDL";
			$i++;
		}
		if ( defined $ENV{rc_np} && $ENV{rc_np} > 0 ) { 
			$robocopy[$i] =  "/NP";
			$i++;
		}
		if ( defined $ENV{rc_logfile} ) { 
			$robocopy[$i] = "/LOG+:\"" . $ENV{rc_logfile} . "\"";
		}
	} else {
		if ( -e "$ENV{CATPRO_REF}\\bin\\robocopy.exe" ) { $robocopy[3] = $ENV{CATPRO_REF} . "\\bin\\robocopy.exe" };
		if ( -e "$ENV{CATPRO_LOC}\\bin\\robocopy.exe" ) { $robocopy[3] = $ENV{CATPRO_LOC} . "\\bin\\robocopy.exe" };
		$i = 6;
		if ( defined $ENV{rc_mir} && $ENV{rc_mir} > 0 ) { 
			$robocopy[$i] =  "/MIR";
			$i++;
		}
		if ( defined $ENV{rc_retry} && $ENV{rc_retry} > 0 ) { 
			$robocopy[$i] =  "/R:" . $ENV{rc_retry};
			$i++;
		}
		if ( defined $ENV{rc_wait} && $ENV{rc_wait} > 0 ) { 
			$robocopy[$i] =  "/W:" . $ENV{rc_wait};
			$i++;
		}
	}
	if ( not defined $robocopy[3] ) {
		print "robocopy.exe not found!";
		exit 1;
	}
}
